@@ -116,6 +116,7 @@ box_database!(
     AVC3SampleEntry                   0x61766333, // "avc3" - Need to check official name in spec.
     AVCConfigurationBox               0x61766343, // "avcC"
     MP4AudioSampleEntry               0x6d703461, // "mp4a"
+    MP4VideoSampleEntry               0x6d703476, // "mp4v"
     ESDBox                            0x65736473, // "esds"
     VP8SampleEntry                    0x76703038, // "vp08"
     VP9SampleEntry                    0x76703039, // "vp09"
@@ -136,4 +137,5 @@ box_database!(
     OriginalFormatBox                 0x66726d61, // "frma"
     MP3AudioSampleEntry               0x2e6d7033, // ".mp3" - from F4V.
     CompositionOffsetBox              0x63747473, // "ctts"
+    LPCMAudioSampleEntry              0x6C70636D, // "lpcm" - quicktime atom
 );
